% Verify IE method against Gillespie's algorithm
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%      Exact numerical integration of the master     %%%%%%%%%%%%
%%%%%%%%%%%        equation in some models of stochastic       %%%%%%%%%%%%
%%%%%%%%%%%                     epidemiology                   %%%%%%%%%%%%
%%%%%%%%%%%              FIGURE GENERATION SOFTWARE            %%%%%%%%%%%%
%%%%%%%%%%%             Authors: Jenkinson/Goutsias            %%%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 01/13/2012             %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function [meanInfected,stdInfected,meanResis,...
             stdResis,meanSusc,stdSusc,time] = gillespie(totalTime)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% INPUT PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% Initialize model parameters
%

s0 = 762;                   % number of susceptibles
i0 = 1;                     % number of infectives
r0 = 0;                     % number of resistant

M = 2;                      % number of reactions

k1 = 0.00218;               % day^-1, S+I -> 2I
k2 = 0.44036;               % day^-1,   I -> R


iter = 1000;                % number of Monte Carlo iterations (>= 2)
time = 0.1:0.1:totalTime;   % simulation time (days)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MONTE CARLO ITERATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

wtbar = waitbar(0,sprintf('Exact Gillespie computation. Please wait ...'));
for i=1:iter
    waitbar(i/iter,wtbar);  
    xinit = [s0 i0 r0]';
    zinit = [0 0]';
    t     = 0;
    tt    = 0;
    Z     = zinit;
    ZZ    = zinit;
    while t < totalTime
        a1   = k1*(s0-Z(1))*(i0+Z(1)-Z(2));
        a2   = k2*(i0+Z(1)-Z(2));
        a    = [a1,a2];
        astr = sum(a);
        if ~astr
            t = totalTime;
        else
            tau  = exprnd(inv(astr));
            m    = find(cumsum(a) > astr*rand, 1 ); 
            t    = t+tau;
            Z(m) = Z(m)+1;
        end
        tt = [tt;t];
        ZZ = [ZZ;Z];
    end
    tt(end)     = totalTime;
    Nz          = size(ZZ);
    N           = Nz(1)/M;
    ZZZ         = reshape(ZZ,M,N);
    ZZZZ(i,1,1) = 0;
    AD = interp1(tt',ZZZ(1,:),time,'nearest');
    Tz = size(AD);
    ZZZZ(i,1,2:Tz(2)) = AD(2:Tz(2));
    AD = interp1(tt',ZZZ(2,:),time,'nearest');
    Tz = size(AD);
    ZZZZ(i,2,2:Tz(2)) = AD(2:Tz(2));
end
close(wtbar);

Zdata = squeeze(ZZZZ);
Xdata = zeros(size(Zdata));
Xdata(:,1,:) = s0 - Zdata(:,1,:);
Xdata(:,2,:) = i0 + Zdata(:,1,:)- Zdata(:,2,:);
Xdata(:,3,:) = r0 + Zdata(:,2,:);

meanSusc =  mean(squeeze(Xdata(:,1,:)),1);
stdSusc  = std(squeeze(Xdata(:,1,:)),1);

meanInfected = mean(squeeze(Xdata(:,2,:)),1);
stdInfected  = std(squeeze(Xdata(:,2,:)),1);

meanResis = mean(squeeze(Xdata(:,3,:)),1); 
stdResis  = std(squeeze(Xdata(:,3,:)),1);