% Solve SIR model using RIE method with fixed step-size
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%      Exact numerical integration of the master     %%%%%%%%%%%%
%%%%%%%%%%%       equation in some models of stochastic        %%%%%%%%%%%%
%%%%%%%%%%%                    epidemiology                    %%%%%%%%%%%%
%%%%%%%%%%%              RIE METHOD (FIXED STEP-SIZE)          %%%%%%%%%%%%
%%%%%%%%%%%              Authors: Jenkinson/Goutsias           %%%%%%%%%%%%
%%%%%%%%%%%               Last Modified: 01/13/2012            %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize simulation parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tic; % start timer

tau = 0.01; % time-step in days   
tf  = 25;   % simulation time in days

display('Running RIE Method (Fixed Step-Size). Please wait...');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

s0 = 762;            % initial number of susceptible individuals
i0 = 1;              % initial number of infected individuals 
r0 = 0;              % initial number of recoverd individuals 

M = 2;               % number of reactions

k1 = 0.00218;        % 1/day, S + I -> 2I
k2 = 0.44036;        % 1/day,     I -> R

steps = tf/tau;      % number of steps

% determine sample space Z

hyperCubeOrigin = zeros(M,1); % origin of hypercube
hyperCubeExtent = [762;763];  % extent of hypercube
K               = 763*764;    % number of states

% define initial probability 

pPrev    = zeros(K,1);
pPrev(1) = 1;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% find implicit Euler matrix

IminusTauA = genMatrix(k1,k2,s0,i0,K,hyperCubeOrigin,hyperCubeExtent,M,tau);
IminusHalfTauA = genMatrix(k1,k2,s0,i0,K,hyperCubeOrigin,hyperCubeExtent,...
                           M,tau/2);

% perform implicit Euler solver steps

for index = 1:steps
    %
    % Do Euler solver steps
    %
    pNext1 = IminusTauA\pPrev; % matlab automatically performs recursive substitution
    
    pTemp = IminusHalfTauA\pPrev;
    
    pNext2 = IminusHalfTauA\pTemp;
    
    %
    % set new probability via Richardson Extrapolation
    %
    pPrev = (2*pNext2)-pNext1; 
    if min(pPrev) < 0 % ensure that Richardson extrapolation produces a probability vector
        pPrev = pNext2;
    end
    
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Map probability vector to SI plane
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

SIprobs = mapProbs(pNext2,hyperCubeOrigin,hyperCubeExtent,s0,i0);

timeElapsed = toc; % store CPU time

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute L2 error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
load ../SIprobsGroundTruth.mat

FinalError = sqrt(sum(sum((SIprobsTruth-SIprobs).^2)));

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Save results to file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

saveResultsToFile;